--
--  ExtendedPlayerTeleport
--
-- @author  nilBrain @ VertexDezign
-- @date	05/12/2023
-- @node
--
-- Copyright (C) VertexDezign, Confidential, All Rights Reserved.

ExtendedPlayerTeleport = {};

ExtendedPlayerTeleport.TIME_PER_METER = 0.1; --in Seconds
ExtendedPlayerTeleport.COST_PER_METER = 1;

local ExtendedPlayerTeleport_mt = Class(ExtendedPlayerTeleport);

---create new instance
---@param modName string | "FS22_extendedPlayerTeleport"
---@param modDirectory any
---@return table
function ExtendedPlayerTeleport.new(modName, modDirectory)
    local self = setmetatable({}, ExtendedPlayerTeleport_mt)

    self.isServer = g_currentMission:getIsServer();
    self.isClient = g_currentMission:getIsClient();
    self.modName = modName;
    self.modDirectory = modDirectory;


    self.debug = false

	self.samples = {}

	self.currentFuncEvent = nil;
	self.delayFunctionEvents = {};

    self.doTextRender = false;
    self.currenDt = 0;

    return self;
end;

--- onDelete
function ExtendedPlayerTeleport:delete()
    g_soundManager:deleteSamples(self.samples);
end;

---load Samples
function ExtendedPlayerTeleport:loadSamples()
    local xmlFile = XMLFile.load("ExtendedPlayerTeleportSamples", Utils.getFilename("resources/sounds.xml", self.modDirectory));

    if xmlFile ~= nil then
        local soundsNode = getRootNode() -- player.rootNode;
        self.samples.start = g_soundManager:loadSampleFromXML(xmlFile.handle, "player.sounds", "start", self.modDirectory, soundsNode, 1, AudioGroup.ENVIRONMENT, nil, nil);
        self.samples.idle = g_soundManager:loadSampleFromXML(xmlFile.handle, "player.sounds", "idle", self.modDirectory, soundsNode, 1, AudioGroup.ENVIRONMENT, nil, nil);
        self.samples.ending = g_soundManager:loadSampleFromXML(xmlFile.handle, "player.sounds", "ending", self.modDirectory, soundsNode, 1, AudioGroup.ENVIRONMENT, nil, nil);
        xmlFile:delete();
    end;
end;

---update
---@param dt integer
function ExtendedPlayerTeleport:update(dt)
    if self.currentFuncEvent == nil then
        for i, event in ipairs(self.delayFunctionEvents) do
            event.currentDt = event.currentDt + dt;
            self.currenDt = event.time - event.currentDt;

            if event.currentDt >= event.time then
                self.currentFuncEvent = event.func;
                self.doTextRender = false;

                g_currentMission:fadeScreen(-1, 1500, self.endFadeToAlpha, self, {event.func, event.target});
                table.remove(self.delayFunctionEvents, i);
            end;
        end;
    end;


    --on fadeScreen we cannot use the draw function...
    if self.doTextRender then
        local x, y = 0.5, 0.5;
        local message = string.format(g_i18n:getText("remainingTime"), self.currenDt/1000);
        setTextAlignment(RenderText.ALIGN_CENTER);
        setTextBold(true);
        setTextColor(0.2, 0.2, 0.2, 0.7);
        renderText(x - 0.002, y - 0.002, 0.023, message);
        setTextColor(0.5, 0.5, 0.5, 1);
        renderText(x, y, 0.023, message);
    end;
end;

---add event to Teleport Player
---@param func function
---@param target table
---@param distance number
function ExtendedPlayerTeleport:addDelayFunctionEvent(func, target, distance)

    local amount = math.floor(distance * ExtendedPlayerTeleport.COST_PER_METER);
	local enoughMoney = true;

	if amount > 0 then
		enoughMoney = amount <= g_currentMission:getMoney();
	end;

	if not enoughMoney then
		g_gui:showInfoDialog({
			text = g_i18n:getText(ShopConfigScreen.L10N_SYMBOL.NOT_ENOUGH_MONEY_BUY);
		});
	else
		local text = string.format(g_i18n:getText("doYouWantDriveTaxi"), g_i18n:formatMoney(amount, 0, true, true));

		g_gui:showYesNoDialog({
			text = text,
			callback = self.onYesNoBuy,
			target = self,
            args = {distance=distance, func=func, target=target, amount=amount}
		});
	end;
end;

---callback from YesNoDialog
---@param yes boolean
---@param args table
function ExtendedPlayerTeleport:onYesNoBuy(yes, args)
	if yes then
        local distance = args.distance;
        local func= args.func;
        local target = args.target;
        local amount = args.amount;

        g_gui:changeScreen(nil);
        g_currentMission.isPlayerFrozen = true;

        local time = distance * ExtendedPlayerTeleport.TIME_PER_METER*1000
        g_currentMission:fadeScreen(1, 2500, self.endFadeToBlack, self, {func, target});

        self:setMoneyChange(amount, g_currentMission.player.farmId);

        table.insert(self.delayFunctionEvents, {func=func, target=target, time=time, currentDt=0});

        g_soundManager:playSample(self.samples.start);
    end;
end

---on beginning of Teleport Event
---@param arguments table
function ExtendedPlayerTeleport:endFadeToBlack(arguments)
    local func = arguments[1];
    local target = arguments[2];
    func(target);

    g_soundManager:playSample(self.samples.idle);

    self.doTextRender = true;
end;

---on end of Teleport Event
---@param arguments table
function ExtendedPlayerTeleport:endFadeToAlpha(arguments)
    g_soundManager:stopSamples(self.samples);
    g_soundManager:playSample(self.samples.ending);

    self.currentFuncEvent = nil;
    g_currentMission.isPlayerFrozen = false;
end;


---Set Money change
---@param amount integer
---@param farmId integer
function ExtendedPlayerTeleport:setMoneyChange(amount, farmId)
    if self.isServer then
        g_currentMission:addMoney(-amount, farmId, MoneyType.OTHER, true, true);
	else
		g_client:getServerConnection():sendEvent(ExtendedPlayerTeleportMoneyEvent.new(amount, farmId));
	end;
end;